
/* 
 * Saibamaze, 10/3/2020
 */


#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <Adafruit_NeoPixel.h>

//PUMP
const int PUMP = 11;

//BUTTONS
const int button3Pin = 4; 
const int button2Pin = 3;
const int button1Pin = 2;
int button3State = 0;
int button2State = 0;
int button1State = 0;
#define ACTIVATED LOW

//LIGHT
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(24, 12, NEO_GRB + NEO_KHZ800);

//SCREEN
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

//GRAPHICS
const unsigned char plant [] PROGMEM = {
0xff, 0xff, 0xf0, 0xff, 0xff, 0xf0, 0xc3, 0xe3, 0xf0, 0x81, 0xc0, 0xf0, 0x00, 0x00, 0x70, 0x81, 
0x40, 0x30, 0xc3, 0x70, 0x10, 0xff, 0x3c, 0x00, 0xff, 0xbe, 0x00, 0xff, 0xbf, 0x90, 0xff, 0xbf, 
0xf0, 0xff, 0x9f, 0xf0, 0xff, 0xdf, 0xf0, 0xff, 0xdf, 0xf0, 0xff, 0xdf, 0xf0, 0xff, 0xdf, 0xf0, 
0xff, 0xdf, 0xf0, 0xff, 0xdf, 0xf0, 0xff, 0x8f, 0xf0, 0xff, 0x07, 0xf0 
};


//BUTTON CONTROL
int cursorPos = 0;

void setup() {
  //BUTTONS
  pinMode(button1Pin, INPUT);
  pinMode(button2Pin, INPUT);
  pinMode(button3Pin, INPUT);
  digitalWrite(button1Pin, HIGH);
  digitalWrite(button2Pin, HIGH);
  digitalWrite(button3Pin, HIGH);
  randomSeed(analogRead(0));

  //PUMP  
  pinMode(PUMP, OUTPUT);
  digitalWrite(PUMP, HIGH);

  //SCREEN
  Serial.begin(115200);
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3D for 128x64
    Serial.println(F("SSD1306 allocation failed"));
    for(;;);
  }
  display.clearDisplay();
  display.setTextColor(WHITE); 

  //LIGHT
  pixels.begin();

  //STARTUP
  display.setTextSize(1);
}

//Water Moisture elvel to detemrine watering
int moistureThreshold = 650;

//Boolean to see if in grow process or not (if continue watering/lighting)
bool isGrowing = false;

//Counter and function used as a safeguard to prevent over watering. If the pump waters 4 times and the sensor doesnt detect, it will pause for 4 hours before relying on the sensor to ater again.
int safeCount = 1;
unsigned long fourHours = 14400000;
void safeGuard(){ 
  if(safeCount >= 4){
    delay(fourHours);
    safeCount = 0;
  } else {
    safeCount += 1;
  }
}

void loop() {
  //NOTES
  //if(button1State == ACTIVATED; This is what you use for button push
  //display.drawBitmap(0, 0, myBitmap, 128, 64, WHITE); // display.drawBitmap(x position, y position, bitmap data, bitmap width, bitmap height, color) TO DRAW PICTURES
  //display.drawBitmap(30, 30, plant, 20, 20, 1); // display.drawBitmap(x position, y position, bitmap data, bitmap width, bitmap height, color)

  display.clearDisplay();
  
  button1State = digitalRead(button1Pin); //Button 1 = middle
  button2State = digitalRead(button2Pin); //Button 3  = left
  button3State = digitalRead(button3Pin); //Button 2 = right

  if(isGrowing){//Light Control
    for(int i=0;i<24;i++){
     pixels.setPixelColor(i, pixels.Color(255,255,255)); // Turns light ON and sets them all to the brightest white
     pixels.show();
    }
  } else {
    for(int i=0;i<24;i++){
     pixels.setPixelColor(i, pixels.Color(0,0,0)); // Turns light OFF
     pixels.show();
    }
  }


  if(isGrowing){ //IN GROW STATE (Lights on, watering based on moisture sensor 
    
     if(button1State==ACTIVATED)//Exit growing
      isGrowing = false;
      delay(150);
   
    if((analogRead(A0)-100) > moistureThreshold){ //AnalogRead(A0) gets the current soil moisture. 300 = very wet, 630 = bone dry. If the sensor reads beyond the threshold (meaning tis dry) it will water for a period of time
      digitalWrite(PUMP, LOW); 
      delay(1000);
      digitalWrite(PUMP, HIGH);
      safeGuard(); //Calls a function in case the sensor doesnt work to prevent over watering
    }

     //Menu
     display.drawBitmap(55, 0, plant, 20, 20, 1);
     display.setCursor(0,25);
     display.print("Moisture Level: ");
     display.print(analogRead(A0)-100); //NOTE: Notice the read in value is -100, that is because it increases by 100 when the light turns on due to current. So we must subtract 100 to keep consistant with before.
     display.setCursor(0,35);
     display.print("Threshold: ");
     display.print(moistureThreshold);
     display.setCursor(0,55);
     display.print("      [EXIT GROW]");
     
    
  } else {//NOT IN GROW STATE, LIGHT OFF AND SETTING SOIL MOISTURE VALUE
    
    if(button1State==ACTIVATED)//Enter growing
      isGrowing = true;
      delay(150);
    
    //Menu
     display.drawBitmap(55, 0, plant, 20, 20, 1);
     display.setCursor(0,25);
     display.print("Moisture Level: ");
     display.print(analogRead(A0));
     display.setCursor(0,35);
     display.print("Threshold: ");
     display.print(moistureThreshold);
     //display.setCursor(0,45);
     //display.print("+100 to threshold");
     display.setCursor(0,55);
     display.print("[-] [ENTER GROW]  [+]");

     if(button3State==ACTIVATED){
      moistureThreshold -= 10;
      delay(100);
     }

     if(button2State==ACTIVATED){
      moistureThreshold += 10;
      delay(100);
     }
    
  }
  
 display.display();
}
